-- name: Minecraft Music 4.0
-- description: \\#40ffff\\Plays background music from Minecraft during gameplay.\n\n\\#ff40ff\\Commands:\n\\#40ff40\\/skip \\#ffffff\\- \\#ffff40\\Skips the current track and plays the next one.\n\\#40ff40\\/song \\#ffffff\\- \\#ffff40\\Displays the current track and shows the time remaining until the next track starts.\n\\#40ff40\\/songlist \\#ffffff\\- \\#ffff40\\Lists all songs with their ID, name, and filename.\n\\#40ff40\\/skipto <Song-ID> \\#ffffff\\- \\#ffff40\\Skips to the specified track by its ID.

_G.MCM_time_left = 60
_G.MCM_audio_stream_minecraft = nil
_G.MCM_song_ids_set = false
_G.MCM_current = nil
_G.MCM_current_id = nil
_G.MCM_next_song = nil

_G.MCM_songs = {
    {id = 0, file = "calm1.mp3", name = "Minecraft", length = 263},
    {id = 0, file = "calm2.mp3", name = "Clark", length = 204},
    {id = 0, file = "calm3.mp3", name = "Sweden", length = 232},
    {id = 0, file = "hal1.mp3", name = "Subwoofer Lullaby", length = 212},
    {id = 0, file = "hal2.mp3", name = "Living Mice", length = 191},
    {id = 0, file = "hal3.mp3", name = "Haggstrom", length = 194},
    {id = 0, file = "hal4.mp3", name = "Danny", length = 258},
    {id = 0, file = "nuance1.mp3", name = "Key", length = 64},
    {id = 0, file = "nuance2.mp3", name = "Oxygène", length = 72},
    {id = 0, file = "piano1.mp3", name = "Dry Hands", length = 71},
    {id = 0, file = "piano2.mp3", name = "Wet Hands", length = 92},
    {id = 0, file = "piano3.mp3", name = "Mice on Venus", length = 283}
}

_G.MCM_get_random_number = function(min, max)
    if not _G.MCM_random_initialized then
        local initial_seed = get_time() * get_global_timer()
        math.randomseed(initial_seed)
        _G.MCM_random_initialized = true
    end

    local random_number
    repeat
        random_number = math.random(min, max)
    until random_number ~= _G.MCM_previous_number

    _G.MCM_previous_number = random_number
    return random_number
end


_G.MCM_on_update_music = function(m)
    if not _G.MCM_song_ids_set then
        for i, song in ipairs(_G.MCM_songs) do
            song.id = i
        end
        _G.MCM_song_ids_set = true
    end

    if not _G.MCM_current then
        local rndInit = _G.MCM_get_random_number(1, #_G.MCM_songs)
        _G.MCM_current_id = rndInit
        _G.MCM_current = _G.MCM_songs[rndInit]
    else
        if m.playerIndex == 0 then
            stop_background_music(get_current_background_music())
            if _G.MCM_time_left < 1 then
                local rndNext = _G.MCM_next_song or _G.MCM_get_random_number(1, #_G.MCM_songs)
                _G.MCM_current_id = rndNext
                _G.MCM_current = _G.MCM_songs[rndNext]
                _G.MCM_next_song = nil
                _G.MCM_time_left = (_G.MCM_current.length * 30) + _G.MCM_get_random_number(300, 600)
                _G.MCM_audio_stream_minecraft = audio_stream_load(_G.MCM_current.file)
                djui_popup_create("\\#ff8080\\Now Playing:  \\#ffff40\\C418 - " .. _G.MCM_current.name, 1)
                audio_stream_play(_G.MCM_audio_stream_minecraft, false, 1)
            else
                _G.MCM_time_left = _G.MCM_time_left - 1
            end
        end
    end
end

function MCM_on_mario_update(m)
    _G.MCM_on_update_music(m)
end

function MCM_on_cmd_skip()
    if _G.MCM_audio_stream_minecraft then
        audio_stream_stop(_G.MCM_audio_stream_minecraft)
    end
    _G.MCM_time_left = 30
    return true
end

function MCM_on_cmd_song()
    djui_chat_message_create("\\#ff4040\\Current Song:  \\#ffff40\\C418 - " .. _G.MCM_current.name .. " \\#40ff40\\[Song-ID: " .. tostring(_G.MCM_current.id) .. "] [File-Name:  " .. tostring(_G.MCM_current.file) .. "] \\#40ffff\\[Next song in:  " .. tostring(math.ceil(_G.MCM_time_left / 30)) .. " seconds]")
    return true
end

function MCM_on_cmd_songlist()
    djui_chat_message_create("\\#40ffff\\<List of all Minecraft Songs>\\#ffffff\\")
    for _, song in ipairs(_G.MCM_songs) do
        djui_chat_message_create("\\#ff4040\\[" .. song.id .. "] \\#ffff40\\" .. song.name .. " \\#40ff40\\(" .. song.file .. ")")
    end
    return true
end

function MCM_on_cmd_skipto(songid)
    local songid_num = tonumber(songid)
    if not songid_num then
        djui_chat_message_create("\\#ff4040\\Invalid Song ID: " .. tostring(songid))
        return true
    end

    songid_num = math.floor(songid_num)
    if songid_num < 1 or songid_num > #_G.MCM_songs then
        djui_chat_message_create("\\#ff4040\\Song ID out of range: " .. tostring(songid_num))
        return true
    end

    _G.MCM_next_song = songid_num
    if _G.MCM_audio_stream_minecraft then
        audio_stream_stop(_G.MCM_audio_stream_minecraft)
    end
    _G.MCM_time_left = 30
    return true
end

function MCM_on_cmd_testrandomness()
    local counts = {}
    for i = 1, #_G.MCM_songs do
        counts[i] = 0
    end

    for _ = 1, 1000 do
        local selected = _G.MCM_get_random_number(1, #_G.MCM_songs)
        counts[selected] = counts[selected] + 1
    end

    djui_chat_message_create("\\#40ffff\\<Test Results (1000 iterations)>\\#ffffff\\")
    for i, count in ipairs(counts) do
        djui_chat_message_create("\\#ff4040\\[" .. i .. "] \\#ffff40\\" .. _G.MCM_songs[i].name .. " \\#40ff40\\- Selected: " .. count .. " times")
    end
    return true
end

hook_event(HOOK_MARIO_UPDATE, MCM_on_mario_update)
hook_chat_command('skip', "Skip to the next track", MCM_on_cmd_skip)
hook_chat_command('song', "Display the current track and time remaining until the next track", MCM_on_cmd_song)
hook_chat_command('songlist', "Lists all songs with their ID, name, and filename", MCM_on_cmd_songlist)
hook_chat_command('skipto', "<Song-ID>", MCM_on_cmd_skipto)
hook_chat_command('testrandomness', "Test song selection frequency over 1000 iterations", MCM_on_cmd_testrandomness)
